@@ -12,6 +12,7 @@ from typing import Any, Dict, List, Tuple, Union
 
 from .annotation import Annotation, AnnotationResult
 from .invariants import invariant
+# from ..utils import check_values_equal
 
 
 class Value(Annotation):
@@ -67,6 +68,25 @@ class Value(Annotation):
     def children(self):
         return []
 
+    def to_dict(self) -> Dict[str, Any]:
+        """
+        Converts this annotation's details to a JSON-friendly dictionary format.
+
+        Output dictionary contains the annotation's name, group, limit number, success message, and
+        failure message, as well as an ``invariants`` key with a list of the names of all invariants
+        used in this value annotation and a ``tol`` key with this annotation's tolerance. This 
+        dictionary does *not* contain the value being tracked.
+
+        Returns:
+            ``dict[str, object]``: the dictionary representation of this annotation
+        """
+        d = super().to_dict()
+        d.update({
+            "invariants": [inv.__name__ for inv in self.invariants],
+            "tol": self.tol,
+        })
+        return d
+
     def check(self, observed_values: List[Tuple[Any, int]]) -> AnnotationResult:
         """
         Checks that the value tracked by this annotation occurs in the list of observed values.
@@ -89,6 +109,22 @@ class Value(Annotation):
 
         first_satisfier = satisfied.index(True)
         return AnnotationResult(True, self, observed_values[first_satisfier][0], observed_values[first_satisfier][1])
+    
+    def __eq__(self, other: Any) -> bool:
+        """
+        Checks whether this annotation is equal to another object.
+
+        To be equal to a ``Value``, the other object must also be a ``Value`` object, have the same
+        set of invariants, the same tolerance, and the same initial value.
+
+        Args:
+            other (``object``): the object to compare to
+
+        Returns:
+            ``bool``: whether the objects are equal
+        """
+        return isinstance(other, type(self)) and self.invariants == other.invariants and \
+            self.check_values_equal(self.initial_value, other.initial_value) and self.tol == other.tol
 
     def _check_observed_value(self, observed_value: Tuple[Any, int]) -> bool:
         """
@@ -111,50 +147,57 @@ class Value(Annotation):
             for other_value in other_values:
                 # if type(value) != type(other_value):
                 #     continue
-
-                # else:
-                if hasattr(value, 'shape'):
-                    try:
-                        if value.shape != other_value.shape:
-                            continue
-                    except AttributeError:
-                        continue
-
-                try:
-                    ub, lb = value + self.tol, value - self.tol
-                    numeric = True
-                
-                except:
-                    numeric = False
-
-                if numeric:
-                    try:
-                        res = np.logical_and(ub >= other_value, other_value >= lb)
-                    except (ValueError, TypeError) as e:
-                        continue
-                else:
-                    try:
-                        if (hasattr(value, "shape") and hasattr(other_value, "shape") and value.shape != other_value.shape) \
-                                or (hasattr(value, "shape") ^ hasattr(other_value, "shape")):
-                            continue
-                        res = value == other_value
-                    except (ValueError, TypeError) as e:
-                        continue
-
-                if isinstance(res, Iterable):
-                    try:
-                        res = all(res)
-                    except ValueError as e:
-                        if isinstance(res, np.ndarray):
-                            res = res.all()
-                        else:
-                            raise e
-
-                if res:
+                if self.check_values_equal(value, other_value, self.tol):
                     return True
 
         return False
 
+    @staticmethod
+    def check_values_equal(value, other_value, tol=0):
+        """
+        """
+        if hasattr(value, 'shape'):
+            try:
+                if value.shape != other_value.shape:
+                    return False
+            except AttributeError:
+                return False
+
+        try:
+            ub, lb = value + tol, value - tol
+            numeric = True
+        
+        except:
+            numeric = False
+
+        if numeric:
+            try:
+                res = np.logical_and(ub >= other_value, other_value >= lb)
+            except (ValueError, TypeError) as e:
+                return False
+        else:
+            try:
+                if (hasattr(value, "shape") and hasattr(other_value, "shape") and value.shape != other_value.shape) \
+                        or (hasattr(value, "shape") ^ hasattr(other_value, "shape")):
+                    return False
+                res = value == other_value
+            except (ValueError, TypeError) as e:
+                return False
+
+        if isinstance(res, Iterable):
+            try:
+                res = all(res)
+            except ValueError as e:
+                if isinstance(res, np.ndarray):
+                    res = res.all()
+                else:
+                    raise e
+
+        if res:
+            return True
+
+        return False
+
 
 class _AttrValue(Value):
     """
@@ -177,6 +220,23 @@ class _AttrValue(Value):
         val = getattr(obj, attr)
         super().__init__(val, **kwargs)
     
+    def __eq__(self, other: Any) -> bool:
+        """
+        Checks whether this annotation is equal to another object.
+
+        To be equal to a ``_AttrValue``, the other object must also be a ``_AttrValue`` object, have
+        equal underlying ``Value`` annotations, have equal source objects, and have the same 
+        ``_attr``.
+
+        Args:
+            other (``object``): the object to compare to
+
+        Returns:
+            ``bool``: whether the objects are equal
+        """
+        return super().__eq__(other) and self.check_values_equal(self._object, other._object) and \
+            self._attr == other._attr
+    
     def check(self, observed_values: List[Tuple[Any, int]]) -> AnnotationResult:
         """
         Checks whether any of the values in ``observed_values`` has an attribute matching the value
@@ -214,22 +274,68 @@ class Attribute(Annotation):
     """
 
     _annotations: List[_AttrValue]
+    _invariants: List[invariant]
+    _tol: float
 
     def __init__(self, obj: Any, attrs: Union[str, List[str]], **kwargs):
         if isinstance(attrs, str):
             attrs = [attrs]
         if not isinstance(attrs, list) or not all(isinstance(a, str) for a in attrs):
             raise TypeError(f"Invalid type for argument 'attrs': {type(attrs)}")
-                
+
+        name = kwargs.pop("name", None)
+        success_message = kwargs.pop("success_message", None)
+        failure_message = kwargs.pop("failure_message", None)
         self._annotations = []
         for attr in attrs:
             if not hasattr(obj, attr):
                 raise AttributeError(f"{obj} has not attribute '{attr}'")
             self._annotations.append(_AttrValue(obj, attr, **kwargs))
+        
+        self._invariants = kwargs.pop("invariants", [])
+        self._tol = kwargs.pop("tol", 0)
+    
+        super().__init__(name=name, success_message=success_message, failure_message=failure_message,
+            **kwargs)
 
     @property
     def children(self):
         return self._annotations
+    
+    def __eq__(self, other: Any) -> bool:
+        """
+        Checks whether this annotation is equal to another object.
+
+        To be equal to an ``Attribute``, the other object must also be an ``Attribute`` object and
+        have equal child annotations.
+
+        Args:
+            other (``object``): the object to compare to
+
+        Returns:
+            ``bool``: whether the objects are equal
+        """
+        return isinstance(other, type(self)) and self.children == other.children
+
+    def to_dict(self) -> Dict[str, Any]:
+        """
+        Converts this annotation's details to a JSON-friendly dictionary format.
+
+        Output dictionary contains the annotation's name, group, limit number, success message, and
+        failure message, as well as an ``invariants`` key with a list of the names of all invariants
+        used in this value annotation, a ``tol`` key with this annotation's tolerance, and a key for 
+        the attributes being checked. This dictionary does *not* contain the value being tracked.
+
+        Returns:
+            ``dict[str, object]``: the dictionary representation of this annotation
+        """
+        d = super().to_dict()
+        d.update({
+            "invariants": [inv.__name__ for inv in self._invariants],
+            "tol": self._tol,
+            "attributes": [av._attr for av in self._annotations],
+        })
+        return d
 
     def check(self, observed_values: List[Tuple[Any, int]]) -> AnnotationResult:
         """
